# A jQuery jqPlot Widget
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# Authors: Ralph Bean <ralph.bean@gmail.com>
#
# (Inspired by Luke Macken's FlotWidget)

from base import jquery_js
from random import random
from tw.api import Widget, JSLink, CSSLink, js_function

jQuery      = js_function('$')
excanvas_js = JSLink(modname=__name__, filename='static/javascript/excanvas.js')
jqplot_css              = CSSLink(modname=__name__,
    filename='static/css/jquery.jqplot.css')
jqplot_js               = JSLink(modname=__name__,
    filename='static/javascript/jquery.jqplot.js')
jqplot_dateAxis_js      = JSLink(modname=__name__,
    filename='static/javascript/jqplugins/jqplot.dateAxisRenderer.min.js')
jqplot_canvasText_js    = JSLink(modname=__name__,
    filename='static/javascript/jqplugins/jqplot.canvasTextRenderer.min.js')
jqplot_canvasAxis_js    = JSLink(modname=__name__,
    filename='static/javascript/jqplugins/jqplot.canvasAxisTickRenderer.min.js')
jqplot_highlighter_js   = JSLink(modname=__name__,
    filename='static/javascript/jqplugins/jqplot.highlighter.min.js')
jqplot_cursor_js        = JSLink(modname=__name__,
    filename='static/javascript/jqplugins/jqplot.cursor.min.js')

class JQPlotWidget(Widget):
    """ A versatile and expandable jQuery plotting widget.

    Using jqPlot, a pure Javascript plotting library for jQuery,
    this widget produces client-side graphical plots of arbitrary datasets
    on-the-fly.  jqPlot is inspired by and similar to flot and is designed
    to be expanded.

    From the the jqPlot homepage:
        "Computation and drawing of lines, axes, shadows even the grid
        itself is handled by pluggable "renderers". Not only are the plot
        elements customizable, plugins can expand functionality of the plot
        too! There are plenty of hooks into the core jqPlot code allowing for
        custom event handlers, creation of new plot types, adding canvases to
        the plot, and more!"

    For detailed documentation of the jqPlot API, visit the jqPlot project
    homepage:  http://www.jqplot.com/
    """
    template = "genshi:tw.jquery.templates.jqplot"
    params = {
            "data"    : "An array of data series",
            "options" : "Plot options",
            "height"  : "The height of the graph",
            "width"   : "The width of the graph",
            "label"   : "Label for the graph",
            "id"      : "An optional ID for the graph"
    }
    javascript = [jquery_js, jqplot_js, excanvas_js, jqplot_dateAxis_js,
                  jqplot_canvasText_js, jqplot_canvasAxis_js,
                  jqplot_highlighter_js, jqplot_cursor_js]
    css = [jqplot_css]
    height = '300px'
    width = '600px'
    label = ''

    def update_params(self, d):
        super(JQPlotWidget, self).update_params(d)
        if not d.get('data'):
            raise ValueError, "JQPlotWidget must have data to graph"
        if not d.get('id'):
            d['id'] = 'jqplot_%s' % str(int(random() * 999))
        data = d.get('data', [])
        options = d.get('options', {})
        self.add_call(js_function('$.jqplot')( "%s" % d.id, data, options))
        return d

